#!/usr/bin/env python3
# -*- coding: utf-8 -*-


#===================================================
#
# Project      : mlsuite building models or predicting
# Description  : main script
# Usage        : 
# Author       : Suxing Li
# Email        : li.suxing@genecast.com.cn
# Created at   : 2022-06-21
#
#====================================================


from MLsuite import main

if __name__ == "__main__":
    main.Commands()
